;**********************************************************************************
;File: demo.asm
;Collection of a few demo applications for PIC2WS281x interface
;**********************************************************************************

;brief instruction:
;simple - no control:
;Reset while RA5 is low. Rainbow pattern is moved arround through the LED stripe
;
;control via async.serial interface (Startbit, 8 bit data, stopbit, no parity):
;connect TX line from host with RA5. After Reset, first send a single char
;0x55 ("U") for automatic baud rate detection.
;Next data byte selects application:
;
;0: Serial-to-WS281x interface mode
;1: individual control of up to 80 LEDs
;2: moving rainbow pattern with various options 
;3 = blue waves
;4 = patternmovetest


#define RBSPEED 0x20			;default speed for moving rainbow-pattern through chain

;*******************************************************************************************
; Program comes here after initialzation of peripherals:
user_main				
	banksel	PORTA
 ifndef SIMULATOR
	btfss	PORTA,5
	goto	rainbow			;pin RA5 held low during reset: show the rainbow...
 endif

;--------------- Setup USART:
	banksel	TXSTA
	bsf	TXSTA,BRGH		;Async.mode, high speed Baud rate generator
	bsf	BAUDCON,BRG16		;16 bit baudrate generator
	bsf	RCSTA,CREN		;enable receiver
	bsf	RCSTA,SPEN		;enable serial port

 ifdef	SIMULATOR
	clrf	SPBRGH
	movlw	8
	movwf	SPBRGL
 else
	bsf	BAUDCON,ABDEN		;enable auto baud detect
	btfsc	BAUDCON,ABDEN		;wait for auto-baud detection complete
	goto	$-1
 endif
	movf	RCREG,W			;clr RCIF, set from ABD

	movlw	1	
	subwf	SPBRGL,F		;--BRG for max.accuracy
	btfss	STATUS,C
	decf	SPBRGH,F

	movf	SPBRGH,W
	btfss	STATUS,Z		
	goto	skipbaud		;BRGH >0: low baudrate

	movf	SPBRGL,W
	addlw	-10			
	movlw	7			;very high BR might not be accurately detected,
	btfss	STATUS,C		;1MBd is the only useful very high baudrate
	movwf	SPBRGL			;-> set accurate value for 1 MBd

skipbaud	

 ifdef	SIMULATOR
	goto	patternmovetest
;	goto	rainbow
 endif

;Initialization of peripherals (incl. USART) complete!
;************************************************************************************
;wait for command:
;00 = send all further serial data to WS2811 immediately
;01 = control single LEDs (4 bytes data packets: LED-ID, G, R, B)
;02 = demo "rainbow", serial data control speed
;03 = waves
;04 = patternmovetest

wtcmd
	call	chkusart
	btfss	PIR1,RCIF
	goto	$-1			;did not receive any data

	banksel	RCREG
	movf	RCREG,W			;read command code
	movwf	temp
	addlw	-(cmdtablend-cmdtablstart)
	btfsc	STATUS,C		;check for valid cmd
	goto	wtcmd			;unsupported: wait for new cmd
	movf	temp,W 
	brw
cmdtablstart
	goto	serial2ws		
	goto	singleledctrl
	goto	rainbow	
	goto	wavepattern
	goto	patternmovetest	
cmdtablend


;****************************************************************************
;cmd 0: send received data to ws281x (via buffered transmission)
;USART interface speed must be 1 MBaud, no breaks within data packet! 
;true output speed is 10.5 s/byte, so TX-FIFO will fill up by 
;1 byte per 20 transfered bytes! I.e. without taking extra measures, 240 bytes FIFO size 
;is big enough for data packets of > 4500 Bytes (= controlling 1500 WS281x).
;This already includes some possible variations of CPU clock frequency.   

serial2ws
	call	chkusart
	btfss	PIR1,RCIF
	goto 	$-1			;no data available

	banksel	RCREG
	movf	RCREG,W			;read data from USART rx-fifo
	call	send2buf
	
	btfsc	flags,TXRUN		;transmission active?
	goto	serial2ws		;y: just get next data
	
	movf	ptr_rd,W		;W = # of data bytes in buffer
	subwf	ptr_wr,W		
	btfss	STATUS,C
	addwf	bufsize,W		;(rollover of ptr_wr)

	addlw	-3			;do not start transmission if less than 3 bytes
	btfsc	STATUS,C		;to transmit
	call	start_tx
	goto	serial2ws

;****************************************************************************
;cmd 1: control first N LEDs, where N depends on size of data buffer.
;The LED chain is continously refreshed from buffer contents.
;Setting of each LED's colour by data packets of following format (4 bytes/pkt):
;<led-#> <g> <r> <b> 
;if led-# = 0xFF, sets colour for all N leds
;breaks > 2ms between bytes reset the rx-state, incomplete packets are ignored

singleledctrl
	bsf	flags,ATXEN		;enable automatic transmission
	call	start_tx		;initiate transmission

ledctrl_lp
	call	chkusart		;expect LED selection
	btfss	PIR1,RCIF
	goto 	$-1			;no serial data available

	banksel	RCREG
	movf	RCREG,W			;read data from USART rx-fifo
	movwf	FSR0L

	call	wait4data
	btfsc	STATUS,C
	goto	ledctrl_lp		;timeout: start new data packet	
	movwf	v_green	

	call	wait4data
	btfsc	STATUS,C
	goto	ledctrl_lp		;timeout: start new data packet		
	movwf	v_red	

	call	wait4data
	btfsc	STATUS,C
	goto	ledctrl_lp		;timeout: start new data packet		
	movwf	v_blue

	incfsz	FSR0L,W			;common colour for all LEDs?
	goto	single_led		;no

;apply new colour to all N LEDs:
	movlw	MAXBUFSIZE/3
	movwf	temp			;loop counter
	clrf	FSR0L			;FSR0 = start of data buffer
setall_lp
	movf	v_green,W
	movwi	FSR0++
	movf	v_red,W
	movwi	FSR0++
	movf	v_blue,W
	movwi	FSR0++
	decfsz	temp,F
	goto	setall_lp

	goto	ledctrl_lp		;next command

;apply new colour to selected LED: 	
single_led
	addlw	-((MAXBUFSIZE/3)+1)	;index beyond buffer?
	btfsc	STATUS,C
	goto	ledctrl_lp		;y: ignore and wait for next command

	rlf	FSR0L,W
	addwf	FSR0L,F			;= FSR0L * 3
	movf	v_green,W
	movwi	FSR0++
	movf	v_red,W
	movwi	FSR0++
	movf	v_blue,W
	movwi	FSR0++
	goto	ledctrl_lp		;next command

;------------------------ Subroutine: check for RX overrun, select PIR1 bank
chkusart
	banksel	RCSTA
	btfss	RCSTA,OERR
	goto	no_overrun

	bcf	RCSTA,CREN		;reset overrun flag
	bsf	RCSTA,CREN		;re-enable receiver

no_overrun
	banksel	PIR1			
	return

;------------------------ Subroutine:
;wait for new data. returns C-flag = 1 if timeout occured, else C = 0, W = data
;total cycles: 10 if RCREG is not empty 
wait4data
	call	chkusart
	btfss	PIR1,RCIF
	goto 	wait4dat1		;no serial data available yet

rx_read_data
	banksel	RCREG
	movf	RCREG,W			;read data from USART rx-fifo
	bcf	STATUS,C		;flag: no timeout
	return

wait4dat1
	call	sys_tm_adj
	movlw	2			;load timer: 2 ms
	addwf	sys_time,W
	movwf	w16

wait4dat2
	call	chkusart
	btfsc	PIR1,RCIF
	goto 	rx_read_data		;serial data available!

	banksel	BAUDCON
	btfss	BAUDCON,RCIDL
	goto	wait4dat2		;receiving data in prograss: don't check for timeout

	call	sys_tm_adj
	movf	w16,W			
	subwf	sys_time,W		
	btfsc	WREG,7			;timeout?
	goto 	wait4dat2		;not yet
	bsf	STATUS,C		;C = timeout-flag
	return

;***********************************************************************************
wavepattern
	MOVLW_LIN pp_ctrl0		
	banksel	pp_data
	movwf	pp_data			;setup data pointer for control data of layer 0
	movwf	FSR0L			;FSR0 -> control data

	movlw	1<<WRAPEN | 1<<TILE	;pattern shall wrap arround end -> start of chain
	movwf	INDF0			

	movlw	15
	movwi	pm_speed[FSR0]		
	movlw	BM_ADD
	movwi	pdp_blendmode[FSR0]	

	movlw	LOW wv1_data
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH wv1_data
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW wv1_end		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH wv1_end
	movwi	pdp_srcend+1[FSR0]

 if N_LAYERS > 1
;setup pattern control data for layer 1:
	MOVLW_LIN pp_ctrl1		
	movwf	pp_data+1		;setup data pointer for control data of layer 1
	movwf	FSR0L			;FSR0 -> control data

	movlw	1<<WRAPEN | 1<<TILE	;pattern shall wrap arround end -> start of chain
	movwf	INDF0			

	movlw	22
	movwi	pm_speed[FSR0]		

	movlw	BM_ADD
	movwi	pdp_blendmode[FSR0]

	movlw	LOW wv1_data
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH wv1_data
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW wv1_end		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH wv1_end
	movwi	pdp_srcend+1[FSR0]
 endif

 if N_LAYERS > 2
;setup pattern control data for layer 2:
	MOVLW_LIN pp_ctrl2		
	movwf	pp_data+2		;setup data pointer for control data of layer 2
	movwf	FSR0L			;FSR0 -> control data

	movlw	1<<WRAPEN | 1<<TILE	;pattern shall wrap arround end -> start of chain
	movwf	INDF0			

	movlw	256-20
	movwi	pm_speed[FSR0]		
	movlw	BM_ADD
	movwi	pdp_blendmode[FSR0]	

	movlw	LOW wv2_data
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH wv2_data
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW wv2_end		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH wv2_end
	movwi	pdp_srcend+1[FSR0]
 endif


;-------------------- infinite loop
wave_lp	
	call	chkusart
	btfsc	PIR1,RCIF		
	call	rb_cmddata		;got some new USART data 
	
	call 	patternprocessor	
	goto	wave_lp			;repeat...



;***********************************************************************************
;test....
patternmovetest
	banksel	bckgr_red
	movlw	0x10
	movwf	bckgr_red

;setup pattern control data for layer 0:
	movlw	PP_CTRL0_LIN		
;	banksel	pp_data
	movwf	pp_data			;setup data pointer for control data of layer 0
	movwf	FSR0L			;FSR0 -> control data

	movlw	1<<TILE | 0<<WRAPEN
	movwi	pdp_flags[FSR0]

	movlw	0xc0
	movwi	pm_speed[FSR0]	
	movlw	BM_OPAQUE
	movwi	pdp_blendmode[FSR0]	

	movlw	0
	movwi 	pdp_substep[FSR0]

	movlw	0
	movwi 	pdp_gap[FSR0]

	movlw	low 0
	movwi 	pdp_startled[FSR0]
	movlw	high 0
	movwi 	pdp_startled+1[FSR0]

	movlw	LOW short_rgb
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH short_rgb
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW short_rgb_end		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH short_rgb_end
	movwi	pdp_srcend+1[FSR0]

 if N_LAYERS > 1
;setup pattern control data for layer 1:
	MOVLW_LIN pp_ctrl1		
;	banksel	pp_data
	movwf	pp_data+1		;setup data pointer for control data of layer 1
	movwf	FSR0L			;FSR0 -> control data

	movlw	8
	movwi	pm_speed[FSR0]	
	movlw	BM_MULTIPLY
	movwi	pdp_blendmode[FSR0]	

	movlw	1
	movwi 	pdp_substep[FSR0]

	movlw	LOW short_rgb
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH short_rgb
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW short_rgb_end		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH short_rgb_end
	movwi	pdp_srcend+1[FSR0]
 endif



;-------------------- infinite loop
test_lp	
	call	chkusart
	btfsc	PIR1,RCIF		
	call	rb_cmddata		;got some new USART data 
	
	call 	patternprocessor	
	goto	test_lp		;repeat...


	

;***********************************************************************************
;Demo: shift rainbow colour pattern through all LEDs
;speed may be adjusted by serial data (+/- 128 * 2 Steps/s)
 
rainbow

;setup pattern control data for layer 0:
	movlw	PP_CTRL0_LIN		
	banksel	pp_data
	movwf	pp_data			;setup data pointer for control data of layer 0
	movwf	FSR0L			;FSR0 -> control data

	movlw	1<<WRAPEN		;pattern shall wrap arround end -> start of chain
	movwf	INDF0			;reset all other flags

	movlw	RBSPEED
	movwi	pm_speed[FSR0]		;set default speed

	movlw	LOW rb_data
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH rb_data
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW rb_dataend		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH rb_dataend
	movwi	pdp_srcend+1[FSR0]

	movlw	BM_UST_ADD
	movwi	pdp_blendmode[FSR0]

;	movlw	0
;	movwi	pdp_startled[FSR0]
;	movwi	pdp_startled+1[FSR0]

;	movlw	(1<<WRAPEN) + (1<<TILE)
;	movlw	(1<<TILE)
;	movwf	INDF0

 if N_LAYERS > 1
;setup pattern control data for layer 1:
	MOVLW_LIN pp_ctrl1		
	movwf	pp_data+1		;set pointer for layer 1 -> control data
	movwf	FSR0L			;FSR0 -> control data

	MOVLW_LIN pp_ctrl1		
	movwf	FSR0L			;FSR0 -> control data
	movlw	1<<WRAPEN		;pattern shall wrap arround end -> start of chain
	movwf	INDF0			;reset all other flags

	movlw	-RBSPEED
	movwi	pm_speed[FSR0]		;set default speed

	movlw	LOW rb_data
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH rb_data
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW rb_dataend		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH rb_dataend
	movwi	pdp_srcend+1[FSR0]

	movlw	BM_UST_ADD
	movwi	pdp_blendmode[FSR0]
 endif

 if N_LAYERS > 2
;setup pattern control data for layer 2:
	MOVLW_LIN pp_ctrl2		
	movwf	pp_data+2
	movwf	FSR0L			;FSR0 -> control data
	movlw	1<<WRAPEN		;pattern shall wrap arround end -> start of chain
	movwf	INDF0			;reset all other flags

	movlw	2 * RBSPEED
	movwi	pm_speed[FSR0]		;set default speed

	movlw	LOW rb_data
	movwi	pdp_srcstart[FSR0]	;source pattern data start
	movlw	HIGH rb_data
	movwi	pdp_srcstart+1[FSR0]

	movlw	LOW rb_dataend		
	movwi	pdp_srcend[FSR0]	;source pattern data end
	movlw	HIGH rb_dataend
	movwi	pdp_srcend+1[FSR0]

	movlw	BM_UST_ADD
	movwi	pdp_blendmode[FSR0]
 endif

	call	patternmoverinit	;reset/initialize timing

;-------------------- infinite loop
rainbow_lp	
	call	chkusart
	btfsc	PIR1,RCIF		
	call	rb_cmddata		;got some new USART data 
	
	call 	patternprocessor	
	goto	rainbow_lp		;repeat...


;--------------- received new command
;Command format: first byte=command, 2nd byte parameter
;lower nibble = layer number (0..2)
;higher nibble = command select:
;	0x = speed data (parameter: speed)
;	1x = select pattern 
;		param: 	00 = rainbow
;			01 = GRB
;	2x = set mode
;		param.: bit 0 = wrap, bit 1 = tile
;	3x = set pattern startpos. relative to beginning of LED chain
;	4x = set pattern startpos. relative to end of LED chain
;	5x = set blend mode
;		param.0..3: opaque, mix, add, subtract, 
;		param.4..6: microstep-opaque, microstep-add, multiply

 
;	6x = set gap
;
;	Fx = set background colour, here lower nibble = channel selection (G|R|B)
;		param: data for selected channel
rb_cmddata
	banksel	RCREG
	movf	RCREG,W			;read command byte
	banksel	rb_command
	movwf	rb_command

	addlw	-0xF0
	addlw	0xF0
	btfsc	STATUS,C
	goto	rb_layercmd		;not a "set background" command

;set background colour value: 
	andlw	0x0F			;0/1/2 = G/R/B select
	addlw	-3
	btfsc	STATUS,C
	return				;invalid channel selection (> 2)!

	TDM2LIN bckgr_grn		;sets asm.-variable "LIN_ADDR" 
	addlw	LIN_ADDR+3
	movwf	FSR0L			;FSR0 -> background colour data
	call	wait4data
	btfss	STATUS,C		;(skip if timeout)
	movwf	INDF0			;got parameter data
	return
		
rb_layercmd	
	andlw	0x0F			
	addlw	-N_LAYERS
	addlw	N_LAYERS
	btfss	STATUS,C
	return				;invalid layer number (>= N_LAYERS

	andlw	0x0F			;W = layer number
 	TDM2LIN pp_data			;set asm-var LIN_ADDR 
	addlw	LIN_ADDR
	movwf	FSR0L			;FSR0 -> pp_data

	movlw	0x0F
	andwf	rb_command,W		;= layer number
	call	get_layerdataddr	;get address of layer's control data
	movwf	INDF0			
	movwf	FSR0L			;FSR0 -> control data

rb_cmdwtpar
	call	wait4data		;wait for command's parameter data
	btfsc	STATUS,C
	return				;got no parameter data

	movwf	temp			;= parameter data
	banksel rb_command
	swapf	rb_command,W
	andlw	0x07
	brw				;jump to prg.code for cmd 
	goto	rb_setspeed		;0: set speed (-128...+127) 
	goto	rb_selectpt		;1: select source pattern
	goto	rb_setmode		;2: set mode (wrap, tile)
	goto	rb_setstartled1		;3: set startled at start of chain
	goto	rb_setstartled2		;4: set startled at end of chain
	goto	rb_setblendmode		;5: set blendmode
	goto	rb_setgap		;6: set gap
	return				;7

;returns (linear) address of layer's data set:
;input: W = layer 
get_layerdataddr
	brw
;while loop generates code for all layers:
 TDM2LIN pp_ctrl0
 variable i=0
 while i < N_LAYERS

	retlw	i*PATTBLKLEN+LIN_ADDR		
i += 1
 endw

;==== execute command: set new speed for moving pattern
rb_setspeed
	movf	temp,W
	movwi	pm_speed[FSR0]		;set new speed data
	return

;==== execute command: set mode (wrap/tile = bit 0/1)
rb_setmode
	movf	temp,W
	xorwf	INDF0,W
	andlw	3		;change only the valid bits!
	xorwf	INDF0,F
	return

;==== execute command: set blend mode 
rb_setblendmode
	movf	temp,W
	addlw	bm_table-bm_tableend
	btfsc	STATUS,C	
	return			;parameter > number of modes

	movf	temp,W
	call	bm_tableread
	movwi	pdp_blendmode[FSR0]
	return

bm_tableread
	brw
bm_table
	retlw	BM_OPAQUE
	retlw	BM_MIX
	retlw	BM_ADD
	retlw	BM_SUBTRACT
	retlw	BM_UST_OPAQUE
	retlw	BM_UST_ADD
	retlw	BM_MULTIPLY
bm_tableend


;==== execute command: set gap parameter
rb_setgap
	movf	temp,W
	movwi	pdp_gap[FSR0]
	movlw	0
	movwi	pdp_gap+1[FSR0]
	return

;==== execute command: set startled at beginning of chain
rb_setstartled1
	movf	temp,W
	movwi	pdp_startled[FSR0]
	movlw	0
	btfsc	temp,7
	movlw	-1		;expand sign bit
	movwi	pdp_startled+1[FSR0]
	return

;==== execute command: set startled at end of chain
rb_setstartled2
	movlw	low CHAINLENGTH
	addwf	temp,W
	movwi	pdp_startled[FSR0]

	movlw	high CHAINLENGTH
	btfsc	temp,7
	movlw	(high CHAINLENGTH)-1	;expand sign bit
	btfsc	STATUS,C
	addlw	1
	movwi	pdp_startled+1[FSR0]
	return

;==== execute command: select another source pattern
rb_selectpt
	movlw	(pt_addrtb_end-pt_addrtb)/4	;number of available patterns
	subwf	temp,W
	btfsc	STATUS,C
	return				;invalid pattern ID

	lslf	temp,F
	lslf 	temp,F			;pattern-ID *4
	addfsr	FSR0,pdp_srcstart	;FSR0 = pdp_srcstart(pattern data)
rb_spatlp	
	call	patternaddr_sub
	movwi	FSR0++
	incf	temp,F
	movlw	3
	andwf	temp,W
	btfss	STATUS,Z
	goto	rb_spatlp		
	return

patternaddr_sub
	movf	temp,W
	brw
pt_addrtb
	retlw	low rb_data	;ID 0: rainbow pattern data
	retlw	high rb_data
	retlw	low rb_dataend
	retlw	high rb_dataend

	retlw	low testpattern	;ID 1: testpattern
	retlw	high testpattern
	retlw	low tp_end
	retlw	high tp_end

	retlw	low wv1_data	;ID 2: wave pattern 1
	retlw	high wv1_data
	retlw	low wv1_end
	retlw	high wv1_end

	retlw	low wv2_data	;ID 3: wave pattern 2
	retlw	high wv2_data
	retlw	low wv2_end
	retlw	high wv2_end

	retlw	low short_rgb	;ID 4: short test pattern
	retlw	high short_rgb
	retlw	low short_rgb_end
	retlw	high short_rgb_end

pt_addrtb_end



#define RB_BRIGHT 50/100

;	dw	0,0,0
;rb_data dw	0x55,0x55,0x55
;	dw	0xAA,0xAA,0xAA


;rainbow colour pattern data: (green, red, blue)
;pattern-ID = 0
	dw	0,0,0	;case microstepping: for interpolation [-1] -> [0]
rb_data
	dw	0*RB_BRIGHT,	1*RB_BRIGHT,	1*RB_BRIGHT
	dw	0*RB_BRIGHT,	2*RB_BRIGHT,	4*RB_BRIGHT
	dw	0*RB_BRIGHT,	4*RB_BRIGHT,  	9*RB_BRIGHT
	dw	0*RB_BRIGHT,	6*RB_BRIGHT,	16*RB_BRIGHT
	dw	0*RB_BRIGHT,	6*RB_BRIGHT,	25*RB_BRIGHT
	dw	0*RB_BRIGHT,	4*RB_BRIGHT,	36*RB_BRIGHT
	dw	1*RB_BRIGHT,	0*RB_BRIGHT,	49*RB_BRIGHT
	dw	4*RB_BRIGHT,	0*RB_BRIGHT,	64*RB_BRIGHT

	dw	9*RB_BRIGHT,	0*RB_BRIGHT,	81*RB_BRIGHT
	dw	16*RB_BRIGHT,	0*RB_BRIGHT,	100*RB_BRIGHT
	dw	25*RB_BRIGHT,	0*RB_BRIGHT,	121*RB_BRIGHT
	dw	36*RB_BRIGHT,	0*RB_BRIGHT,	100*RB_BRIGHT
	dw	49*RB_BRIGHT,	0*RB_BRIGHT,	81*RB_BRIGHT
	dw	64*RB_BRIGHT,	0*RB_BRIGHT,	64*RB_BRIGHT
	dw	81*RB_BRIGHT,	0*RB_BRIGHT,	49*RB_BRIGHT
	dw	100*RB_BRIGHT,	0*RB_BRIGHT,	36*RB_BRIGHT

	dw	121*RB_BRIGHT,	1*RB_BRIGHT,	25*RB_BRIGHT
	dw	144*RB_BRIGHT,	4*RB_BRIGHT,	16*RB_BRIGHT
	dw	169*RB_BRIGHT,	9*RB_BRIGHT,	9*RB_BRIGHT
	dw	196*RB_BRIGHT,	16*RB_BRIGHT,	4*RB_BRIGHT
	dw	169*RB_BRIGHT,	25*RB_BRIGHT,	1*RB_BRIGHT
	dw	144*RB_BRIGHT,	36*RB_BRIGHT,	0*RB_BRIGHT
	dw	121*RB_BRIGHT,	49*RB_BRIGHT,	0*RB_BRIGHT
	dw	100*RB_BRIGHT,	64*RB_BRIGHT,	0*RB_BRIGHT

	dw	81*RB_BRIGHT,	81*RB_BRIGHT,	0*RB_BRIGHT
	dw	64*RB_BRIGHT,	100*RB_BRIGHT,	0*RB_BRIGHT
	dw	49*RB_BRIGHT,	121*RB_BRIGHT,	0*RB_BRIGHT
	dw	36*RB_BRIGHT,	144*RB_BRIGHT,	0*RB_BRIGHT
	dw	25*RB_BRIGHT,	169*RB_BRIGHT,	0*RB_BRIGHT
	dw	16*RB_BRIGHT,	144*RB_BRIGHT,	0*RB_BRIGHT
	dw	9*RB_BRIGHT,	121*RB_BRIGHT,	0*RB_BRIGHT
	dw	4*RB_BRIGHT,	100*RB_BRIGHT,	0*RB_BRIGHT

	dw	1*RB_BRIGHT,	81*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	64*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	49*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	32*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	16*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	9*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	4*RB_BRIGHT,	0*RB_BRIGHT
	dw	0*RB_BRIGHT,	1*RB_BRIGHT,	0*RB_BRIGHT
rb_dataend

	dw	0,0,6		;(microstepping)
testpattern
	dw	100,0,0
	dw	50,0,0
	dw	25,0,0
	dw	12,0,0
	dw	6,0,0
	dw	0,100,0
	dw	0,50,0
	dw	0,25,0
	dw	0,12,0
	dw	0,6,0
	dw	0,0,100
	dw	0,0,50
	dw	0,0,25
	dw	0,0,12
	dw	0,0,6
tp_end

	dw	0, 0, 0		;(microstepping)
wv1_data
	dw	0, 0, 0
	dw	0, 0, 0
	dw	0, 0, 0
	dw	0, 0, 0
	dw	0, 0, 0
	dw	1, 0, 1
	dw	1, 0, 1
	dw	1, 0, 1
	dw	2, 0, 1
	dw	2, 0, 2
	dw	2, 0, 2
	dw	3, 0, 3
	dw	3, 0, 3
	dw	3, 0, 4
	dw	3, 0, 5
	dw	4, 0, 5
	dw	5, 0, 6
	dw	5, 0, 6
	dw	7, 0, 7
	dw	9, 0, 10
	dw	10, 0, 10
	dw	11, 1, 11
	dw	12, 3, 13
	dw	17, 5, 16
	dw	23, 7, 22
	dw	33, 10, 31
	dw	46, 15, 44
	dw	64, 20, 61
	dw	90, 30, 86
	dw	32, 14, 35
	dw	19, 9, 22
	dw	12, 5, 14
	dw	7, 3, 8
	dw	4, 1, 5
	dw	3, 0, 2
	dw	1, 0, 1
	dw	0, 0, 0
	dw	0, 0, 0
	dw	0, 0, 0
	dw	0, 0, 0
	dw	0, 0, 0


wv1_end	

wv2_data
	dw	12, 0, 7
	dw	49, 12, 43
	dw	57, 16, 51
	dw	90, 30, 86
	dw	43, 8, 36
	dw	24, 1, 17
	dw	12, 4, 13
	dw	11, 2, 11
	dw	10, 0, 10
	dw	9, 0, 10
	dw	8, 0, 7
	dw	7, 0, 7
	dw	6, 0, 7
	dw	5, 0, 6
	dw	4, 0, 5
	dw	3, 0, 4
	dw	2, 0, 3
	dw	1, 0, 2

wv2_end


	dw	0,0,0
short_rgb
	dw	0,0x40,0
;	dw	0,0x40,0
	dw	0x40,0,0
;	dw	0x40,0,0
	dw	0,0,0x40
;	dw	0,0,0x40
	dw	0,0,0
short_rgb_end
