;************************************************************************
; Controlling LEDs/LED stripes with WS2811 controllers or WS2812 LEDs
; 
; Version 0.1			(C) 2014 Thomas Elger (thomas@picalic.de)
; 
; This program is free software; you can redistribute it and/or modify it 
; under the terms of the GNU General Public License as published by the 
; Free Software Foundation; either version 3 of the License, or (at your 
; option) any later version.
; This program is distributed in the hope that it will be useful, but 
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
; or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
; for more details.

; For a copy of GNU General Public License, see <http://www.gnu.org/licenses/>. 
;************************************************************************
;
; This code is written for the following PIC microcontrollers:
; PIC12F1822, PIC12F1840
;_______________________________________________________________
; Pin Usage (8 pin package):
;
; RA0 (Pin 7, out): modulator output (MDOUT) => output signal to WS2811
; RA1 (Pin 6, out): SCK, SPI clock, wire connected with RA2 (MDCIN1)
; RA2 (Pin 5, in): modulator carrier input for "1" signal (MDCIN1 = SPI clock)
; RA3 (Pin 4, in): Reset 
; RA4 (Pin 3, out): SDO, comparator input (comparator used for delay)
; RA5 (Pin 2, in): USART Rx  
;_______________________________________________________________
	radix dec			; Default base = 10 
;___________________________________________________________________________
;****************    Configuration of software build	********************

;#define SIMULATOR	;for simulation with MPLAB sim

;The base level software is code for initialization, tx-FIFO, tx-ISR.
;User application starts at label "user_main".
;Below there is an application example that uses just base level.   

;Uncomment the following #defines in order to extend the base level 
;software by additional functionality:

#define USE_PATPROC		;add pattern processor routines
				;(handles output of data patterns from memory to any
				;location of the LED chain)

#define USE_MICROSTEP		;allow microstep positioning of pattern

#define USE_TIMER		;a few 16-bit wide milliseconds timer routines

#define USE_PATMOVER		;supports moving patterns through the LED chain
				;(requires pattern processor and timer)

#define USE_DEMO		;test and demo, requires all above options!
;____________________________________________________________________________

 ifdef __12F1840
#include <p12F1840.inc>         	; processor specific variable definitions
#define MAXBUFSIZE 240			; max. usable buffer size (3 x 80 bytes)
#define ENDGPREG 0x170			; last addr+1 of usable general purpose registers 
 endif

 ifdef __12F1822
#include <p12F1822.inc>         	; processor specific variable definitions
#define MAXBUFSIZE 112			; max. usable buffer size (80+32 bytes)
#define ENDGPREG 0x0C0			; last addr+1 of usable general purpose registers 
 endif

 __config _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_ON & _MCLRE_ON & _CP_OFF & _BOREN_ON & _IESO_OFF & 0x3FFF
 __config _CONFIG2, _WRT_OFF & _PLLEN_ON & _BORV_HI & _LVP_OFF & 0x3FFF


 errorlevel-302				;avoid lots of warnings about banking

;**************************************************************

#define TX_KBPS 800			;WS281x Transmission speed in kbps
					;choose from: 800, 615, 400, 250 

;#define FIFOSIZE 180			;tx-FIFO size, if defined, must be multiple of 3!
#define SPEEDFACTOR 2			;for patternmover: pattern moving speed is
					;(pm_speed * SPEEDFACTOR) steps/sec. 
#define CHAINLENGTH 288			;# of serially connected WS281X chips
#define N_LAYERS 3			;number of layers (pattern processor)

#define TRUE 1
#define FALSE 0

;======== flags-bits: 
#define ATXEN	0			;ctrl: enable automatic retransmission (ISR)
#define TXRUN	1			;flag: transmission (buffer->LEDs) is running
#define EOCHAIN	2			;flag: led_idx reached end of LED chain
#define GAPFLG 	3			;data ptr = beyond end of pattern

;======== status & control bits for pattern display processor (pdp_flags[FSRx]):
#define WRAPEN	0			;enable wrapping arround at end of chain
#define TILE	1			;tile mode: pattern repeats through whole LED-chain
#define ACTIVE	5			;LED position within current pattern data 
#define WRAPFLG	6			;current data = wrapped part of pattern @start of chain
#define DONE	7			;pattern display finished (LED-index > end of pattern data)
 
;======== Blend modes:
#define BM_ADD pp_add-pp_bmref
#define BM_OPAQUE pp_opaque-pp_bmref
#define BM_SUBTRACT pp_subtract-pp_bmref
#define BM_MIX pp_mix-pp_bmref
#define BM_MULTIPLY pp_multiply-pp_bmref
 ifdef USE_MICROSTEP
#define BM_UST_OPAQUE pp_ustep_opaque-pp_bmref
#define BM_UST_ADD pp_ustep_add-pp_bmref
 endif

#include macros.asm
	
;**************************************************************
;RAM data
;**************************************************************
;LED/FIFO buffer begins always at address = 0x20
;linear addressing is used for indirect addressing of buffer through (FSR0)
;therefore always keep FSR0H = 0x20! (Even ISR relies on that)

;------------------------- RAM usage if pattern processor is used:
; start of RAM (0x20)
; 	FIFO (linear addr. 0 = traditional 0x20)
; 	pattern control data, layer 0
; 	pattern control data, layer 1
; 	...
; 	pattern control data, layer N
; 	gp_regs variables (-> upper available bank)
; ENDGPREG = end of usable RAM+1 (adress depends on C-type)
;
;  	0x70..0x7F (all banks): common RAM variables
;--------------  


;common RAM data (linear FIFO addressing automatically skips this block!)
	CBLOCK 0x70
flags					;various control bits and status-flags
ptr_rd					;data buffer ptr (read)
ptr_wr					;data buffer ptr (write)
bufsize					;buffer size
temp					;general purpose, temporary storage 
					;(subroutines may change it's contents!)
sys_time: 2				;increments every millisecond
sys_ts					;(used for generation of millisecond intervals)
w16: 2					;16 bit general purpose working register
led_idx: 2				;currently processed LED (0..CHAINLENGTH-1)

;variables for individual LED control (used by DEMO): 
v_green
v_red
v_blue

commonregsizecheck: 0
	ENDC

 if commonregsizecheck > 0x80 
	error "not enough common RAM!"
 endif 

 ifdef USE_PATMOVER
PMV equ 1
 else 
PMV equ 0
 endif 

 ifdef USE_MICROSTEP
UST equ 1
 else 
UST equ 0
 endif 

;----------------------------------------------------------------------
;Definition of data structure for pattern display processor+pattern mover:
	CBLOCK 0
pdp_flags			;some control/status flags
pm_speed: PMV			;speed for moving pattern (-128..+127)
pm_taccu: PMV			;(used for move timing)
pdp_substep: UST		;for microstepping
pdp_startled: 2			;LED number, where pattern starts
pdp_gap: 2			;gap between patterns in tile mode
pdp_srcstart: 2			;pointer to start of source pattern
pdp_srcend: 2			;pointer to end of source pattern (+1)
pdp_srcptr: 2			;currently processed source pattern data
pdp_blendmode			;blend mode
PATTBLKLEN:0			;(to get length of this block)
	ENDC
;---------------------------------------------------------------------- 
 ifdef USE_PATPROC
PPV equ 1
 else 
PPV equ 0
 endif 

#define GPREGSIZE 25			;reserved GP-register space for variables

	CBLOCK ENDGPREG-GPREGSIZE

gp_regs: 0				;to define base of following registers

;--- (put any other user registers here)---

;used by pattern processor:
temp_grn: PPV
temp_red: PPV
temp_blue: PPV
pp_grn: PPV
pp_red: PPV
pp_blue: PPV
pp_data: N_LAYERS*PPV			;pointer (LSB of linear addr.) to pattern data structure 
					;for each layer
bckgr_grn: PPV				;if defined: background colour 		
bckgr_red: PPV				;(if these labels are not defined, LEDs outside
bckgr_blue: PPV				;of any pattern will stay dark)

ptr_wr_alt: PPV				;alternate write pointer (temporary used)
pp_layer: PPV
pp_accu: 2*PPV				;another 16 bit storage

pm_timer: 2*PMV				;used by pattern mover

;other registers (not pattern processor): 
rb_command				;(for Demo)

gpregsizecheck: 0
	ENDC


	TDM2LIN gp_regs			;get linear address of gp_regs
GP_REGS_LIN equ LIN_ADDR		;working with linear addresses...
PP_CTRL0_LIN equ GP_REGS_LIN - N_LAYERS * PATTBLKLEN

	
 if gpregsizecheck > ENDGPREG
	error "not enough GP-registers reserved - please increase GPREGSIZE!"
 endif 

 ifdef FIFOSIZE
  if (MAXBUFSIZE-FIFOSIZE) < GPREGSIZE
	error "FIFO too large - please decrease FIFOSIZE!"
  endif 

  if (FIFOSIZE %3) != 0
	error "FIFOSIZE has to be multiple of 3!"
  endif
 endif

;***************************************************************************
;RESET entry - LowLevel.asm must be first file with program code!
#include LowLevel.asm			;ISR, hardware-initialization
					; & some more low level routines
					;(the basic level stuff)

 ifdef USE_PATPROC
#include PatternProcessor.asm	
 endif	

 ifdef USE_PATMOVER
#include PatternMover.asm	
 endif	

 ifdef USE_TIMER
#include Timer.asm
 endif

 ifdef USE_MICROSTEP
#include microstep.asm
 endif

 ifdef USE_DEMO
#include demo.asm
 else

;***************************************************************************
;Example Application (may be replaced by user-application):
;
;Sends data from USART Rx to ws281x (via FIFO), so this is just a little
;"UART-to-WS281x" converter. UART interface speed is 1 MBaud.  
;
;Remarks:
;True output speed is 10.5 s/byte, so TX-FIFO will fill up by 1 byte per 
;20 transfered bytes! I.e. without taking extra measures, 240 bytes FIFO size 
;is big enough for data packets of > 4500 Bytes (= controlling 1500 WS281x).
;This already includes some possible variations of CPU clock frequency.  
;
;Does not use any other variables than from common area (0x70..0x7F),
;therefore bufsize = MAXBUFSIZE (from init) is used. 

;=============== entry point after initialization:
user_main

;--------------- Setup USART:
	banksel	TXSTA
	bsf	TXSTA,BRGH		;Async.mode, high speed Baud rate generator
	bsf	BAUDCON,BRG16		;16 bit baudrate generator
	bsf	RCSTA,CREN		;enable receiver
	bsf	RCSTA,SPEN		;enable serial port

	clrf	SPBRGH
	movlw	7		
	movwf	SPBRGL			;set SPBRG value for 1 MBd


;------- infinite loop:
serial2ws
	banksel	PIR1
	btfss	PIR1,RCIF
	goto 	$-1			;no data available

	banksel	RCREG
	movf	RCREG,W			;read data from USART rx-fifo
	call	send2buf
	
	btfsc	flags,TXRUN		;transmission active?
	goto	serial2ws		;y: just get next data
	
	movf	ptr_rd,W		;W = # of data bytes in buffer
	subwf	ptr_wr,W		
	btfss	STATUS,C
	addwf	bufsize,W		;(rollover of ptr_wr)

	addlw	-3			;do not start transmission if less than 3 bytes
	btfsc	STATUS,C		;to transmit
	call	start_tx
	goto	serial2ws

 endif ;(USE_DEMO)

 ifdef USE_PATPROC
#include multiply.asm
 endif				
	END
